/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsIDOMNode;

/**
 * A service used to sort the contents of a XUL widget.
 */
[scriptable, uuid(F29270C8-3BE5-4046-9B57-945A84DFF132)]
interface nsIXULSortService : nsISupports
{
    const unsigned long SORT_COMPARECASE = 0x0001;
    const unsigned long SORT_INTEGER = 0x0100;

    /**
     * Sort the contents of the widget containing <code>aNode</code>
     * using <code>aSortKey</code> as the comparison key, and
     * <code>aSortDirection</code> as the direction.
     *
     * @param aNode A node in the XUL widget whose children are to be sorted.
     * @param aSortKey The value to be used as the comparison key.
     * @param aSortHints One or more hints as to how to sort:
     *
     *   ascending: to sort the contents in ascending order
     *   descending: to sort the contents in descending order
     *   comparecase: perform case sensitive comparisons
     *   integer: treat values as integers, non-integers are compared as strings
     *   twostate: don't allow the natural (unordered state)
     */
    void sort(in nsIDOMNode aNode,
              in AString aSortKey,
              in AString aSortHints);
};

%{C++
nsresult
NS_NewXULSortService(nsIXULSortService **result);
%}
